#include <cassert>
#include <stdexcept>
#include <iostream>
#include "xunix.h"
#include "xutil.h"

namespace {

  const char* progname;

  int async_to_sync_pipe[2];

  void signal_catcher(int sig) {
    // write() is async-signal-safe.
    xunix::write(async_to_sync_pipe[1], &sig, sizeof(sig));
  }

}

int main(int argc, char* argv[])
{
  int rv = 0;

  progname = xutil::basename(argv[0]);

  try {

    //
    // Install a signal handler for all available signals . . .
    //
#ifdef linux
    int nsig = _NSIG;
#elif sgi
    int nsig = _sys_nsig;
#endif 
    for( int i = 1 ; i < nsig ; ++i ) {
      // except SIGKILL AND SIGSTOP.
      if( (i == SIGKILL) || (i == SIGSTOP) ) {
        continue;
      }
      xutil::install_signal_handler(i, &signal_catcher);
    }

    //
    // Set up the usual pipe.
    //
    xunix::pipe(async_to_sync_pipe);

    //
    // Wait for the next signal.
    //
    fd_set rset;
    int max_fd = async_to_sync_pipe[0] + 1;
    for( ;; ) {
      FD_ZERO(&rset);
      FD_SET(async_to_sync_pipe[0], &rset);
      try {
        xunix::select(max_fd, &rset, 0, 0, 0);
      } catch( const xunix::exception xue ) {
        if( xue.error_number() == EINTR ) {
          continue;
        }
        throw;
      }

      //
      // The next signal was received.  Print it.
      //

      int sig;
      xunix::read(async_to_sync_pipe[0], &sig, sizeof(sig));
#ifdef linux
      char* signal_description = strsignal(sig);
#elif sgi
      char* signal_description = _sys_siglist[sig];
#endif
      std::cout << progname << ": "
                << "signal(" << sig << "): " << signal_description
                << std::endl;
    }
    
  } catch( const std::exception& e ) {
    std::cerr << progname << ": " << e.what() << std::endl;
    rv = 1;
  } catch( ... ) {
    std::cerr << progname << ": Caught unexpected exception." << std::endl;
    rv = 1;
  }

  return rv;
}
